@@ -31,14 +31,15 @@
 #' @param transform a character vector pointing to a function that should be applied to both
 #'  axes of the normal quantile plot. The most common use is to set \code{transform="pnorm"}
 #'  for a PP-plot.
-#' @param sim.method How should residuals be simulated? The default \code{"refit"} option constructs new responses (via \code{\link{simulate}}),
+#' @param sim.method How should residuals be simulated? The default for most objects is \code{"refit"}, which constructs new responses (via \code{\link{simulate}}),
 #' refits the model (via \code{\link{update}}), then recomputes residuals, often known as a \emph{parametric bootstrap}.
 #' This is computationally intensive but gives a robust answer. This is the only suitable option if
 #' residuals are not expected to be normal when assumptions are satisfied (like when using \code{\link[stats]{glm}} with a non-Gaussian family). 
 #' Alternatively, \code{"norm"} simulates from a normal distribution, matches means and variances (and covariances for multivariate responses) to the observed residuals.
 #' The \code{"stand.norm"} option sets means to zero and variances to one, which is appropriate when residuals
 #' should be standard normal when assumptions are satisfied (as for any object fitted using the \code{mvabund}
-#' package, for example). These options are faster but more approximate than \code{"refit"}.
+#' package, for example). These options are faster but more approximate than \code{"refit"}, in fact \code{"stand.norm"} is 
+#' used as the default for \code{\link[mvabund]{manyglm}} objects, for computational reasons.
 #' @param main the plot title (if a plot is produced). A vector of three titles, one for each plot.
 #'  If only one value is given that will be used for all plots.
 #' @param xlab \code{x} axis label (if a plot is produced). A vector of three labels, one for each plot.
@@ -46,15 +47,18 @@
 #' @param ylab \code{y} axis label (if a plot is produced). A vector of three labels, one for each plot.
 #'  If only one value is given that will be used for all plots.
 #' @param col color of points
-#' @param line.col color of the line on diagnostic plots about which points should show no trend if model assumptions are satisfied.
-#' Defaults to "olivedrab". Because it's cool. If \code{do.smooth=TRUE} this is the color of the smoother (defaulting to "slateblue4").
+#' @param line.col a vector of length three containing the colors of the lines on the three diagnostic plots.
+#' Defaults to "slateblue4" for smoothers and to "olivedrab" otherwise. Because it's cool.
 #' @param envelope.col color of the global envelope around the expected trend. All data points should always stay within this envelope
 #' (and will for a proportion \code{conf.level} of datasets satisfying model assumptions). If a smoother has been used on
 #' the residual vs fits or scale-location plot, the envelope is constructed around the smoother, that is, the smoother should always stay
 #' within the envelope.
-#' @param do.smooth logical defaults to \code{FALSE} (no smoother). If \code{TRUE}, a smoother is drawn on residual vs fits and
-#' scale-location plots, and the global envelope is around the \emph{smoother} not the data. No smoother is added to the normal quantile plot.
+#' @param add.smooth logical defaults to \code{TRUE}, which adds a smoother to residual vs fits and
+#' scale-location plots, and computes a global envelope around the \emph{smoother} rather than the data (\code{add.smooth=FALSE}). No smoother is added to the normal quantile plot.
 #' @param plot.it logical. Should the result be plotted? If not, a list of analysis outputs is returned, see \emph{Value}.
+#' @param fitMin the minimum fitted value to use in plots, any fitted value less than this will be truncated to
+#' \code{fitMin}. This is useful for generalised linear models, where small fitted values correspond to
+#' predictions that are numerically zero. The default is to set \code{fitMin} to \code{-6} for GLMs, otherwise no truncation (\code{-Inf}).
 #' @param ... further arguments sent through to \code{plot}
 #' 
 #' @details
@@ -72,7 +76,7 @@
 #' }
 #' and see if the trend is behaving as expected if the model were true. As long as 
 #' the fitted model responds to \code{\link{residuals}} and \code{\link{predict}} 
-#' (and when \code{sim.method="refit"}, \code{\link{simulate}}) then a simulation envelope
+#' (and when \code{sim.method="refit"}, \code{\link{simulate}} and \code{\link{update}}) then a simulation envelope
 #' will be constructed for each plot.
 #' 
 #' Simulation envelopes are global, constructed using the \code{\link[GET]{GET-package}}, meaning that
@@ -82,7 +86,7 @@
 #' The \code{\link[GET]{GET-package}} was originally constructed to place envelopes around functions, motivated by
 #' the problem of diagnostic testing of spatial processes (Myllymäki et al 2017), but it can equally
 #' well be applied here, by treating the set of residuals (ordered according to the x-axis) as point-wise evaluations of a function.
-#' For residual vs fits and scale-location plots, global envelopes are constructed for
+#' For residual vs fits and scale-location plots, if \code{do.smooth=TRUE}, global envelopes are constructed for
 #' the \emph{smoother}, not for the data, hence we are looking to see if the smoother
 #' is wholly contained within the envelope. The smoother is constructed using \code{\link[mgcv]{gam}} from the \code{mgcv} package.
 #' 
@@ -90,19 +94,19 @@
 #' assumptions -- they do not tell you whether the violations are large enough to worry about. For example, in linear models,
 #' violations of normality are usually much less important than violations of linearity or equal variance. And in all cases,
 #' modest violations tend to only have modest effects on the validity of inferences, so you need to think about how big
-#' observed violations are rather than just thinking about whether or not they are outside their simulation envelope.
+#' observed violations are rather than just thinking about whether or not anything is outside its simulation envelope.
 #' 
 #' The method used to simulate data for the global envelopes is controlled by \code{sim.method}.
 #' The default method (\code{sim.method="refit"}) uses a \emph{parametric bootstrap} approach: simulate 
 #' new responses from the fitted model, refit the model and recompute residuals and fitted values. 
 #' This directly assesses whether trends in observed trends depart from what would be expected if the fitted model
 #' were correct, without any further assumptions. For complex models or large datasets this would however be super-slow.
-#' A fast, more approximate alternative (\code{sim.method="norm"}) is to  simulate new (multivariate) normal residuals 
+#' A fast, more approximate alternative (\code{sim.method="norm"}) is to simulate new (multivariate) normal residuals 
 #' repeatedly and use these to assess whether trends in the observed data depart from what would be expected
 #' for independent (multivariate) normal residuals. If residuals are expected to be standard
 #' normal, a more refined check is to simulate from the standard normal using (\code{sim.method="stand.norm"}).
-#' This might for example be useful when diagnosing a model fitted using the \code{mvabund} package,
-#' since this calculates Dunn-Smyth residuals, which are approximately standard normal when assumptions are satisfied.  
+#' This might for example be useful when diagnosing a model fitted using the \code{mvabund} package (Wang et al. 2012),
+#' since this calculates Dunn-Smyth residuals (Dunn & Smyth 1996), which are approximately standard normal when assumptions are satisfied.  
 #' If \code{y} is a \code{glm} with non-Gaussian family then residuals will not be normal and \code{"refit"} is the
 #' only appropriate option, hence other choices will be overridden with a warning reporting that this
 #' has been done. 
@@ -122,20 +126,27 @@
 #' 
 #' @return Up to three diagnostic plots with simulation envelopes are returned, and additionally a list of 
 #' three objects used in plotting, for plots 1-3 respectively. Each is a list with five components:\describe{
-#' \item{\code{x}}{$X$-values used for the envelope. In plots 1 and 3 with \code{do.smooth=TRUE}, this is 500 equally spaced points covering
+#' \item{\code{x}}{\emph{X}-values used for the envelope. In plots 1 and 3 this is the fitted values, or if \code{add.smooth=TRUE}, this is 500 equally spaced points covering
 #' the range of fitted values. For plot 2, this is sorted normal quantiles corresponding to observed data.}
-#' \item{\code{y}}{In plots 1 and 3, this is the values of the smoother corresponding to \code{x}. For plot 2,
+#' \item{\code{y}}{The \emph{Y}-values used for the envelope. In plots 1 and 3 this is the residuals, or with \code{add.smooth=TRUE}, this is the values of the smoother corresponding to \code{x}. For plot 2,
 #' this is the sorted residuals.}
 #' \item{\code{lo}}{The lower bound on the global envelope, for each value of \code{x}.}
 #' \item{\code{hi}}{The upper bound on the global envelope, for each value of \code{x}.}
-#' \item{\code{p.value}}{A \emph{P}-value for the test that the observed smoother or data is not unusually far
-#' from what was expected, computed in \code{\link[GET]{global_envelope_test}}.} 
+#' \item{\code{p.value}}{A \emph{P}-value for the test that observed smoother or data are consistent with what
+#' would be expected if the fitted model were correct, computed in \code{\link[GET]{global_envelope_test}}.} 
 #' }
 #' 
 #' @author David Warton <david.warton@@unsw.edu.au>
 #' @references 
+#' 
+#' Dunn, P. K., & Smyth, G. K. (1996), Randomized quantile residuals. J. Comp. Graphical Stat. 5, 236-244. doi:10.1080/10618600.1996.10474708
+#' 
 #' Myllymäki, M., Mrkvička, T., Grabarnik, P., Seijo, H. and Hahn, U. (2017), Global envelope tests for spatial processes. J. R. Stat. Soc. B, 79: 381-404. doi:10.1111/rssb.12172
 #' 
+#' Wang, Y. I., Naumann, U., Wright, S. T., & Warton, D. I. (2012), \code{mvabund} – an \code{R} package for model‐based analysis of multivariate abundance data. Methods in Ecology and Evolution, 3, 471-474. doi:10.1111/j.2041-210X.2012.00190.x
+#' 
+#' Warton DI (2022) Eco-Stats - Data Analysis in Ecology, from \emph{t}-tests to multivariate abundances. Springer, ISBN 978-3-030-88442-0
+#'
 #' @seealso \code{\link{cpredict}}, \code{\link{cresiduals}}, \code{\link{qqenvelope}}
 #' @examples
 #' # fit a Poisson regression to random data:
@@ -151,21 +162,22 @@
 #' # check normality assumption:
 #' plotenvelope(iris_mlm,n.sim=99,which=2)
 #' 
-#' # A few more plots, with envelopes around smoothers: (this will take several seconds to run)
-#' \donttest{plotenvelope(iris_mlm, which=1:3, do.smooth=TRUE)
+#' # A few more plots, with envelopes around data not smoothers:
+#' \donttest{plotenvelope(iris_mlm, which=1:3, add.smooth=FALSE)
 #' # Note violation on the scale/location plot.}
 #' 
 #' @importFrom grDevices adjustcolor 
-#' @importFrom graphics plot points lines par polygon
-#' @importFrom stats cor fitted formula lm model.frame model.matrix model.response predict qnorm qqnorm quantile residuals residuals.lm rnorm rstandard runif sd update var
-
+#' @import graphics
+#' @import stats
 #' @export
+
 plotenvelope = function (y, which = 1:2, sim.method="refit", 
                        n.sim=199, conf.level=0.95, type="st", transform = NULL, 
                        main = c("Residuals vs Fitted Values", "Normal Quantile Plot", "Scale-Location Plot"), xlab = c("Fitted values", "Theoretical Quantiles", "Fitted Values"),
                        ylab = c("Residuals", "Residuals", expression(sqrt("|Residuals|"))), col=par("col"), 
-                       line.col=if(do.smooth) "slateblue4" else "olivedrab", envelope.col = adjustcolor(line.col, 0.1), do.smooth=FALSE,
-                       plot.it = TRUE, ...) 
+                       line.col=if(add.smooth) c("slateblue4","olivedrab","slateblue4") else rep("olivedrab",3), 
+                       envelope.col = adjustcolor(line.col, 0.1), add.smooth=TRUE,
+                       plot.it = TRUE, fitMin=if(inherits(y,"glm")|inherits(y,"manyglm")) -6 else -Inf, ...) 
 {
   ### which plots to construct? And clean up arguments
   show = rep(FALSE, 6)
@@ -181,8 +193,8 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
     stop("y must be an object obtained by fitting a model to data, it cannot be the dataset itself")
   else
   {
-    object = y
-    y      = model.response(model.frame(object))
+    object=y
+    yResp     = model.response(model.frame(object))
   }
   
   # define residual function to be cresiduals for mlm, rstandard for lm, otherwise residuals
@@ -198,34 +210,46 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
   } 
 
   # define predict function to be cpredict for mlm, otherwise predict
-  predFunction = 
-  if(inherits(object,"mlm")) 
-    cpredict
-  else
-  {
-    predict
-  } 
+  predFunction = if(inherits(object,"mlm")) 
+#    cpredict
+#  else
+#    fitted
+    function(obj, xMin){ pmax(cpredict(obj), xMin) }
+  else 
+    function(obj, xMin){ pmax(predict(obj), xMin) }
+    
+
   y = resFunction(object)
-  x = predFunction(object)
+  x = predFunction(object, fitMin)
+  
+  y[y==Inf] = 2*max(y[y<Inf]) #deal with any probs with infinite resids
+
+  # set defaults for sim.method ("stand.norm" for manyglm, otherwise "refit")
+  if(all(sim.method==c("refit","norm","stand.norm"))) 
+    sim.method = if(inherits(object,"manyglm")) "stand.norm" else "refit"
 
   # if it's a non-Gaussian GLM, change sim.method to "refit" with warning if required
   if(inherits(object,"glm")) 
   {
-    if(sim.method!="refit" & object$family$family!="gaussian")
+    if(object$family$family!="gaussian")
     {
+      if(sim.method[1]!="refit")
+        warning("A GLM for non-Gaussian data has been detected. Residuals will not be Gaussian when the model is correct, so sim.method has been changed to refit")
       sim.method="refit"
-      warning("A GLM for non-Gaussian data has been detected. Residuals will not be Gaussian when the model is correct, so sim.method has been changed to refit")
     }
-  }
+  } 
   
   # check if it is multivariate data, if so set a flag for later and vectorise res
   if(length(dim(y))>1)
   {
     n.resp = dim(y)[2]
     n.rows = dim(y)[1]
-    is.mva = TRUE
-    mu     = switch(sim.method, "stand.norm" = rep(0, n.resp), colMeans(y) )
-    Sigma  = switch(sim.method, "stand.norm" = cor(y), var(y) )
+    if(n.resp>1)
+      is.mva = TRUE
+    else
+      is.mva = FALSE
+    mu     = switch(sim.method[1], "stand.norm" = rep(0, n.resp), colMeans(y) )
+    Sigma  = switch(sim.method[1], "stand.norm" = cor(y), var(y) ) 
     if(col==par("col"))
       col  = rep(1:n.resp,each=n.rows)
   }
@@ -233,31 +257,77 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
   {
     is.mva = FALSE
     n.resp = 1
-    mu     = switch(sim.method,"stand.norm" = 0, mean(y))
-    sigma  = switch(sim.method,"stand.norm" = 1, sd(y))
+    mu     = switch(sim.method[1],"stand.norm" = 0, mean(y))
+    sigma  = switch(sim.method[1],"stand.norm" = 1, sd(y))
   }
   
   ### now simulate new residuals and get fitted values too ###
   n.obs=length(y)
   
   # simulate to get limits around these
-  if(sim.method=="refit")
+  if(sim.method[1]=="refit")
   {
-    modelF = model.frame(object)
-    yNew   = simulate(object,n.sim)
+    # get a model frame with everything in it and update on original data.
+    # This is done to set up the framework to use for simulating - all we would need to do then is change first element of mf (response).
+    if( inherits(object,c("lmerMod","glmerMod")) )
+      mf <- match.call(call=object@call)
+    else
+      mf <- match.call(call=object$call)
+    m <- match(c("formula", "data", "subset", 
+                 "weights", "na.action", "etastart", 
+                 "mustart", "offset"), names(mf), 0L)
+    mf <- mf[c(1L, m)]
+#    mf$drop.unused.levels <- TRUE
+    mf[[1L]] <- quote(stats::model.frame)
+    modelF <- try( eval(mf, parent.frame()), silent=TRUE )
+    
+    # if for some reason this didn't work (mgcv::gam objects cause grief) then just call model.frame on object:    
+    # also, do this for lme4 because it is so not a team player 
+    if(inherits(modelF, "try-error") | inherits(object,c("lmerMod","glmerMod")) )
+      modelF = model.frame(object)
+
+    # if response has brackets in its name, it is some sort of expression,
+    # put quotes around it so it works (?)
+    respName   = names(modelF)[1]
+    if(regexpr("(",respName,fixed=TRUE)>0)
+    {
+      newResp    = sprintf("`%s`", respName) #putting quotes around response name
+      fm.update  = reformulate(".", response = newResp)
+    }
+    else
+      fm.update  = reformulate(".")
+
+        # if there is an offset, add it, as a separate argument when updating
+    offs=NULL
+    modelF$offs=try(model.offset(modelF))
+    if(inherits(modelF$offs,"try-error") | is.null(modelF$offs))
+      objectY = update(object, formula = fm.update, data=modelF)
+    else
+      objectY = update(object, formula = fm.update, data=modelF,offset=offs)
+    
+
+    # simulate new data as a matrix
+    yNew   = simulate(objectY,n.sim)
     if(is.mva) # for multivariate data, vectorise res for each sim dataset
       yNew = apply(yNew,3,cbind)
     resids = fits = matrix(NA,nrow(yNew),ncol(yNew))
-#    logLiks=rep(NA,n.sim)
     for(i.sim in 1:n.sim)
     {
-      modelF[1]      = matrix(yNew[,i.sim],ncol=n.resp,dimnames=dimnames(y))
-      newFit         = try(update(object,data=modelF))
+      modelF[[1]]      = matrix(yNew[,i.sim],ncol=n.resp,dimnames=dimnames(yResp))
+      if(inherits(modelF$offs,"try-error") | is.null(modelF$offs))
+        newFit         = try(update(objectY, formula=fm.update, data=modelF))
+      else
+        newFit         = try(update(objectY, formula=fm.update, data=modelF, offset=offs))
       resids[,i.sim] = resFunction(newFit)
-      fits[,i.sim]   = predFunction(newFit)
-#      logLiks[i.sim] = logLik(newFit)
+      ftt   = try(predFunction(newFit, fitMin)) #using try for fitted values because eel data occasionally failed(!?)
+      if(inherits(ftt,"try-error"))
+        fits[,i.sim] = x
+      else
+        fits[,i.sim] = ftt
+
+      if(inherits(fits[,i.sim],"try-error"))
       # if no variation in preds, add v small random noise to avoid error later
-      if(var(fits[,i.sim])<1.e-8) fits[,i.sim] = fits[,i.sim] + 1.e-6*rnorm(n.obs)
+        if(var(fits[,i.sim])<1.e-8) fits[,i.sim] = fits[,i.sim] + 1.e-6*rnorm(n.obs)
     }
   }
   else  
@@ -274,6 +344,14 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
       resids = matrix(rnorm(n.obs*n.sim,mean=mu,sd=sigma),ncol=n.sim)
     fits = matrix(rep(x,n.sim),ncol=n.sim)
   }
+  
+  # for smoothers, check df is not larger than the model permits
+  if(add.smooth)
+  {
+    nXs = length(unique(x))
+    kStart = if(nXs<11) max(nXs-3,1) else -1 
+  }
+  
   out=vector("list",3)
   
   ### plot 1 - residuals vs fits plot ###
@@ -281,12 +359,18 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
   if (show[1L])
   {
     # get observed smoother
-    if(do.smooth)
+    if(add.smooth)
     {
       nPred = 500
       xPred = seq(min(x),max(x),length=nPred)
       # get smoother for observed data
-      gam.yx=mgcv::gam(c(y)~s(c(x)))
+      if(nXs>3) 
+        gam.yx=mgcv::gam(c(y)~s(c(x),k=kStart))
+      if(nXs==3)
+        gam.yx = lm(c(y)~c(x)+I(c(x^2)))
+      if(nXs==2)  
+        gam.yx=lm(c(y)~c(x))
+      
       resObs=as.vector(predict(gam.yx,newdata=data.frame(x=xPred)))
 
       # get smoothers for simulated data
@@ -294,7 +378,12 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
       for(i.sim in 1:n.sim)
       {
         xiSim=fits[,i.sim]
-        gam.yxi=mgcv::gam(resids[,i.sim]~s(xiSim))
+        if(nXs>3)
+          gam.yxi = mgcv::gam(resids[,i.sim]~s(xiSim,k=kStart))
+        if(nXs==3)
+          gam.yxi = lm(resids[,i.sim]~xiSim+I(xiSim^2))
+        if(nXs==2)
+          gam.yxi = lm(resids[,i.sim]~xiSim)
         residFn[,i.sim]=predict(gam.yxi,newdata=data.frame(xiSim=xPred))
       }
     }
@@ -322,10 +411,15 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
     if(plot.it==TRUE)      #do a res vs fits plot and add sim envelope
     {
       # make axes and scales as appropriate
-      if(do.smooth) #for smoother, keep ylim to data only
+      loPlot = cr$lo
+      hiPlot = cr$hi
+      if(add.smooth) #for smoother, keep ylim to data only
       {
         plot(x,y, main=main[1], 
              xlab=xlab[1], ylab=ylab[1], type="n", ...)
+        eps=0.025*(max(y)-min(y)) # truncate envelope so it just goes a little bit outside the data
+        loPlot[cr$lo<min(y)] = min(y)-eps
+        hiPlot[cr$hi>max(y)] = max(y)+eps
       }
       else #otherwise ylim should cover envelope
       {
@@ -333,13 +427,13 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
              xlab=xlab[1], ylab=ylab[1], type="n", ...)
       }
       # add envelope and line
-      polygon(xPred[c(1:nPred,nPred:1)], c(cr$lo,cr$hi[nPred:1]), 
-              col=envelope.col, border=NA)
+      polygon(xPred[c(1:nPred,nPred:1)], c(loPlot,hiPlot[nPred:1]), 
+              col=envelope.col[1], border=NA)
       # add smooth, if applicable
-      if(do.smooth)
-        lines(xPred,resObs,col=line.col)
+      if(add.smooth)
+        lines(xPred,resObs,col=line.col[1])
       else
-        lines(range(x),c(0,0),col=line.col,...)
+        lines(range(x),c(0,0),col=line.col[1],...)
       # add data
       points(x, y, col=col, ...)
       
@@ -375,16 +469,32 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
         # make qqplot
         plot(qq.x$x, qq.x$y, col=col, main=main[2], 
              xlab=xlab[2], ylab=ylab[2], ...)
-        #add a qq line as is done in qqline, but limited to range of data
+        #add a qq line.
+        # this is done as in qqline, but limited to range of data, unless sim.method="stand.norm"
         probs=c(0.25,0.75)
         qs=quantile(qq.x$y,probs)
         xs=qnorm(probs)
-        slope=diff(qs)/diff(xs)
-        int=qs[1]-slope*xs[1]
-        lines(range(qq.x$x),int+slope*range(qq.x$x),col=line.col)
+        if(sim.method[1]=="stand.norm")
+        {
+          slope = 1
+          int = 0
+        }
+        else
+        {
+          slope=diff(qs)/diff(xs)
+          int=qs[1]-slope*xs[1]
+        }
+        #truncate limits outside of data range so polygon actually bloody plots 
+        loPlot = cr$lo
+        hiPlot = cr$hi
+        eps=0.025*(max(y)-min(y))
+        loPlot[cr$lo<min(y)] = min(y)-eps
+        hiPlot[cr$hi>max(y)] = max(y)+eps
+
         # add envelope
-        polygon(sort(qq.x$x)[c(1:n.obs,n.obs:1)], c(cr$lo,cr$hi[n.obs:1]), 
-                col=envelope.col, border=NA)
+        polygon(sort(qq.x$x)[c(1:n.obs,n.obs:1)], c(loPlot,hiPlot[n.obs:1]), 
+                col=envelope.col[2], border=NA)
+        lines(range(qq.x$x),int+slope*range(qq.x$x),col=line.col[2])
       }
     
       # return a list with qq values and limits, ordered same way as input data
@@ -401,21 +511,31 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
     yAbs    = sqrt(abs(y))
     residsAbs = sqrt(abs(resids))
     # get observed smoother
-    if(do.smooth)
+    if(add.smooth==TRUE)
     {
       nPred = 500
       xPred = seq(min(x),max(x),length=nPred)
       # get smoother for observed data
-      gam.yx=mgcv::gam(c(yAbs)~s(c(x)))
+      if(nXs>3)
+        gam.yx=mgcv::gam(c(yAbs)~s(c(x),k=kStart))
+      if(nXs==3)
+        gam.yx=lm(c(yAbs)~c(x)+I(c(x^2)))
+      if(nXs==2)
+        gam.yx=lm(c(yAbs)~c(x))
       resObs=as.vector(predict(gam.yx,newdata=data.frame(x=xPred)))
       
       # get smoothers for simulated data
       residFn=matrix(NA,nPred,n.sim)
       for(i.sim in 1:n.sim)
       {
-        xiSim          = fits[,i.sim]
-        gam.yxi        = mgcv::gam(residsAbs[,i.sim]~s(xiSim))
-        residFn[,i.sim] = predict(gam.yxi,newdata=data.frame(xiSim=xPred))
+        xiSim            = fits[,i.sim]
+        if(nXs>3)
+          gam.yxi        = mgcv::gam(residsAbs[,i.sim]~s(xiSim,k=kStart))
+        if(nXs==3)
+          gam.yxi        = lm(residsAbs[,i.sim]~xiSim+I(xiSim^2))
+        if(nXs==2)
+          gam.yxi        = lm(residsAbs[,i.sim]~xiSim)
+        residFn[,i.sim]  = predict(gam.yxi,newdata=data.frame(xiSim=xPred))
       }
     }
     else
@@ -442,23 +562,29 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
     if(plot.it==TRUE)      #do a res vs fits plot and add sim envelope
     {
       # make axes and scales as appropriate
-      if(do.smooth) #for smoother, keep ylim to data only
+      loPlot = cr$lo
+      hiPlot = cr$hi
+      if(add.smooth==TRUE) #for smoother, keep ylim to data only
       {
-        plot(x,yAbs, main=main[3], 
+        plot(x, yAbs, main=main[3], 
              xlab=xlab[3], ylab=ylab[3], type="n", ...)
+        #truncate limits outside of data range so polygon actually bloody plots 
+        eps=0.025*(max(yAbs)-min(yAbs))
+        loPlot[cr$lo<min(yAbs)] = min(yAbs)-eps
+        hiPlot[cr$hi>max(yAbs)] = max(yAbs)+eps
       }
       else #otherwise ylim should cover envelope
       {
-        plot(c(x,xPred,xPred), c(yAbs,cr$lo,cr$hi), main=main[3], 
+        plot(c(x,xPred,xPred),c(yAbs,cr$lo,cr$hi), main=main[3], 
              xlab=xlab[3], ylab=ylab[3], type="n", ...)
       }
       # add envelope and line
-      polygon(xPred[c(1:nPred,nPred:1)], c(cr$lo,cr$hi[nPred:1]), 
-              col=envelope.col, border=NA)
+      polygon(xPred[c(1:nPred,nPred:1)], c(loPlot,hiPlot[nPred:1]), 
+              col=envelope.col[3], border=NA)
 #      lines(range(xObs),median(yAbs)*c(1,1),col=line.col,...)
       # add smooth, if applicable
-      if(do.smooth)
-        lines(xPred,resObs,col=line.col,...)
+      if(add.smooth==TRUE)
+        lines(xPred,resObs,col=line.col[3],...)
       # add data
       points(x, yAbs, col=col, ...)
       
@@ -469,7 +595,7 @@ plotenvelope = function (y, which = 1:2, sim.method="refit",
   }
   
   
-  # an envelope around points on res vs fits plot
+  # an envelope around points on res vs fits plot - used in simulation work for faster comp times
   if (show[4L])
   {
     # get observed smoother
